function roll,x,r,rot=rot
; ROLL   Rolls matrix elements.
;    y = ROLL(x,R) rolls matrix x  R elements in dimension .
;    The rolled matrix y is the same size as x. The last element
;    in each dimension is repeated to fill blank places
;
;    Example: ROLL([1, 2, 3, 4],1) = [1, 1, 2, 3]
;             ROLL([1, 2, 3, 4], -1) = [2, 3, 4, 4]
;             ROLL([1,  2,  3,  4] ,$         [1 1 2 3]
;                       [5,  6,  7,  8] ,$           = [1 1 2 3]
;                      [9, 10, 11, 12],[1 1])   [5 5 6 7]
;
;    If x is matrix, ROLL(x,N) == ROLL(x,[N,N])
;
;    ROLL(x,[Ri],/rot) rotates the elements instead of 
;    discarding them.
;      Example: ROLL([1, 2, 3, 4],1,/rot) = [4, 1, 2, 3]
; 
; Verify parameters
if n_elements(x) le 1 then on_error,2
sz=size(x)
ndims=sz[0]
if keyword_set(rot) then rot =1 else rot=0
if ~rot then begin
    case ndims of 
	1: begin
		nx=sz[1]		
		R=sgn(R)*(abs(R) mod sz[1])
		x1=shift(x,R)
		if R ne 0 then begin
		    if R gt 0 then x1[0:R-1]=x1[R] else x1[nx+R:*]=x1[nx-1+R]	
		endif
		y=x1
	    end
	2: begin
		nx=sz[1] & ny=sz[2]
		if n_elements(R) lt 2 then R=replicate(R,2)
		R=sgn(R)*(abs(R) mod sz[1:2])
		x1=shift(x,[R[0],0])
		if R[0] ne 0 then begin
		    if R[0] gt 0 then x1[0:R[0]-1,*]=x1[R[0],*] else x1[nx+R[0]:*,*]=x1[nx-1+R[0],*]
		endif 
		y1=shift(x1,[0,R[1]])
		if R[1] ne 0 then begin
		      if R[1] gt 0 then y1[*,0:R[1]-1]=y1[*,R[1]] else y1[*,ny+R[1]:*]=y1[*,ny-1+R[1]]
		endif 
		y=y1
	    end
	else: on_error,2
    endcase
endif 

if rot then y=shift(x,R)
 
return,y
end